#include "OptionsWindow.h"
#include "VerticalBox.h"
#include "Messages.h"

/* (c) 1999, Tim Vernum
 * Licenced under the Acknowledged Use Licence
 * See Doc/Licence
 */

#include <CheckBox.h>
#include <RadioButton.h>
#include <Button.h>
#include <Screen.h>
#include <MessageFilter.h>

#include <stdio.h>

const uint32 MSG_Cancel = B_CANCEL ;
const uint32 MSG_OK = 'Okay' ;
const uint32 MSG_FilteredKeyDown = '#KYD' ;

class KeyFilter : public BMessageFilter
{
	public:
		KeyFilter( uint32 key )
		:	BMessageFilter( B_KEY_DOWN )
		,	fKey(key)
		{
		}
		
		filter_result Filter( BMessage * msg, BHandler ** )
		{
			uint32 key ;
			if( msg->FindInt32( "raw_char", (int32*)&key ) == B_OK )
			{
				if( key == fKey )
					msg->what = MSG_FilteredKeyDown ;
			}
			return B_DISPATCH_MESSAGE;
		}
 
	private:
		uint32 fKey ;
} ;


WorkBar::OptionsWindow :: OptionsWindow( void )
:	inherited( BRect(75,75,175,175) ,  "Options" , B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_CLOSABLE | B_NOT_ZOOMABLE )
,	fOrientation(2)
,	fLook(4)
,	fFeel(2)
,	fFlags(6)
{
	BRect r(0,0,100,100) ;
	float winWidth = 0 ,
	      groupWidth = 0 ,
	      winHeight = 0 ;

	VerticalBox * box1 ;
	VerticalBox * box2 ;
	BRadioButton * rb ;
	BCheckBox * cb ;

	// Background
	BView * bg = new BView( r, "bg", B_FOLLOW_ALL_SIDES, B_WILL_DRAW ) ;
	bg->SetViewColor( ui_color( B_PANEL_BACKGROUND_COLOR ) ) ;
	AddChild( bg ) ;

	// Column 1, Box 1	
	r.top = 3 ;
	box1 = new VerticalBox( r, "orientation" ) ;
	bg->AddChild( box1 ) ;
	box1->SetLabel( "Orientation" ) ;
	
	r.left = 10 ;
	r.top = 15 ; 
	rb = new BRadioButton( r, "vertical", "Vertical", NULL ) ;
	box1->AddView( rb ) ;
	fOrientation.AddButton( rb ) ;

	rb = new BRadioButton( r, "horizontal", "Horizontal", NULL ) ;
	box1->AddView( rb ) ;
	fOrientation.AddButton( rb ) ;

	box1->ResizeToPreferred( ) ;
	r = box1->Frame() ;
		
	if( r.right > groupWidth )
		groupWidth = r.right ;

	r.top = r.bottom + 2 ;
	r.bottom += 10 ;
	
	// Column 1, Box 2	
	box2 = new VerticalBox( r, "look" ) ;
	bg->AddChild( box2 ) ;
	box2->SetLabel( "Look" ) ;
	
	r.left = 10 ;
	r.top = 15 ; 
	rb = new BRadioButton( r, "regular", "Regular", NULL ) ;
	box2->AddView( rb ) ;
	fLook.AddButton( rb ) ;

	rb = new BRadioButton( r, "minitab", "MiniTab", NULL ) ;
	box2->AddView( rb ) ;
	fLook.AddButton( rb ) ;

	rb = new BRadioButton( r, "tabless", "Tabless", NULL ) ;
	box2->AddView( rb ) ;
	fLook.AddButton( rb ) ;

	rb = new BRadioButton( r, "borderless", "Borderless", NULL ) ;
	box2->AddView( rb ) ;
	fLook.AddButton( rb ) ;

	box2->ResizeToPreferred( ) ;
	r = box2->Frame() ;
		
	if( r.right > groupWidth )
		groupWidth = r.right ;

	if( r.bottom > winHeight )
		winHeight = r.bottom ;

	// Column 1, Sync
	box1->ResizeTo( groupWidth, box1->Frame().Height() ) ;
	box2->ResizeTo( groupWidth, box2->Frame().Height() ) ;


	// Column 2, Box 1	
	r.top = 3 ;
	r.left = groupWidth + 2;
	r.right = r.left + 10;
	groupWidth = 0 ;
	 
	box1 = new VerticalBox( r, "feel" ) ;
	bg->AddChild( box1 ) ;
	box1->SetLabel( "Feel" ) ;
	
	r.left = 10 ;
	r.top = 15 ; 
	rb = new BRadioButton( r, "normal", "Normal", NULL ) ;
	box1->AddView( rb ) ;
	fFeel.AddButton( rb ) ;

	rb = new BRadioButton( r, "floating", "Floating", NULL ) ;
	box1->AddView( rb ) ;
	fFeel.AddButton( rb ) ;

	box1->ResizeToPreferred( ) ;
	r = box1->Frame() ;
		
	if( r.right > winWidth )
		winWidth = r.right ;
	if( r.Width() > groupWidth )
		groupWidth = r.Width() ;

	// Column 1, Box 2	
	r.top = r.bottom + 2 ;
	r.bottom += 10 ;
	
	box2 = new VerticalBox( r, "Flags" ) ;
	bg->AddChild( box2 ) ;
	box2->SetLabel( "Flags" ) ;
	
	r.left = 10 ;
	r.top = 15 ; 
	cb = new BCheckBox( r, "closable", "Closable", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	cb = new BCheckBox( r, "movable", "Movable", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	cb = new BCheckBox( r, "resizable", "Resizable", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	cb = new BCheckBox( r, "avoid_focus", "Avoid Focus", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	cb = new BCheckBox( r, "avoid_front", "Avoid Front", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	cb = new BCheckBox( r, "first_click", "First Click", NULL ) ;
	box2->AddView( cb ) ;
	fFlags.AddBox( cb ) ;

	box2->ResizeToPreferred( ) ;
	r = box2->Frame() ;
		
	if( r.right > winWidth )
		winWidth = r.right ;
	if( r.Width() > groupWidth )
		groupWidth = r.Width() ;
	if( r.bottom > winHeight )
		winHeight = r.bottom ;
		
	// Column 2, Sync	
	box1->ResizeTo( groupWidth, box1->Frame().Height() ) ;
	box2->ResizeTo( groupWidth, box2->Frame().Height() ) ;

	r.left = 15 ;
	r.top = r.bottom + 10 ;
	BButton * cancel = new BButton( r, "cancel", "Cancel", new BMessage(MSG_Cancel) ) ;
	bg->AddChild( cancel ) ;
	cancel->ResizeToPreferred() ;
		
	BButton * ok = new BButton( r, "ok", "OK", new BMessage(MSG_OK) ) ;
	bg->AddChild( ok ) ;
	ok->ResizeToPreferred() ;
	r = ok->Frame() ;
	ok->MoveTo( winWidth - r.Width() - 15, r.top );
	ok->MakeDefault( true ) ;
	
	winHeight = r.bottom + 5 ;

	// Window, Sync
	ResizeTo( winWidth , winHeight ) ;
	
	AddCommonFilter( new KeyFilter(B_ESCAPE) ) ;
}

void WorkBar::OptionsWindow :: Init( BMessage * msg , const BMessenger& tgt, BPoint pt )
{
	fTarget = tgt ;
	
	int32 i32 ;
	msg -> FindInt32( "look", &i32 ) ;
	switch( i32 )
	{
		case B_FLOATING_WINDOW_LOOK:
			fLook.SetSelection( WB_MiniTab ) ;
			break ;

		case B_MODAL_WINDOW_LOOK :
			fLook.SetSelection( WB_Tabless ) ;
			break ;

		case B_BORDERED_WINDOW_LOOK :
			fLook.SetSelection( WB_Borderless ) ;
			break ;

		case B_TITLED_WINDOW_LOOK:
		default:
			fLook.SetSelection( WB_Regular ) ;
			break ;
	}
	
	msg -> FindInt32( "feel", &i32 ) ;
	switch( i32 )
	{
		case B_FLOATING_SUBSET_WINDOW_FEEL :
		case B_FLOATING_APP_WINDOW_FEEL	:
		case B_FLOATING_ALL_WINDOW_FEEL :
			fFeel.SetSelection( WB_Floating ) ;
			break ;
	
		default:
			fFeel.SetSelection( WB_Normal ) ;
			break ;
	}

	msg -> FindInt32( "flags", &i32 ) ;
	uint32 flags = 0;
	if( !(i32 & B_NOT_MOVABLE) )
		flags |= WB_Movable ;
	if( !(i32 & B_NOT_CLOSABLE) )
		flags |= WB_Closable ;
	if( !(i32 & B_NOT_RESIZABLE) )
		flags |= WB_Resizable ;
	if(  (i32 & B_AVOID_FRONT) )
		flags |= WB_AvoidFront ;
	if(  (i32 & B_AVOID_FOCUS) )
		flags |= WB_AvoidFocus ;
	if(  (i32 & B_WILL_ACCEPT_FIRST_CLICK) )
		flags |= WB_AcceptFirst ;
	
	fFlags.SetValue( flags ) ;	

	bool horizontal ;
	msg -> FindBool( "horizontal", &horizontal ) ;
	if( horizontal )
		fOrientation.SetSelection( WB_Horizontal ) ;
	else
		fOrientation.SetSelection( WB_Vertical ) ;

	BRect screenRect ;
	{
		BScreen screen( this ) ;
		screenRect = screen.Frame() ;
	}

	BRect winRect = Bounds() ;
	pt.x -= winRect.right/2 ;
	if( pt.x < 10 )
		pt.x = 10 ;

	pt.y -= winRect.bottom/2 ;	
	if( pt.y < 30 )
		pt.y = 30 ;
		
	if( pt.x > screenRect.right - winRect.right - 15 )
		pt.x = screenRect.right - winRect.right - 15 ;

	if( pt.y > screenRect.bottom - winRect.bottom - 15 )
		pt.y = screenRect.bottom - winRect.bottom - 15 ;

	MoveTo( pt ) ;
}

void WorkBar::OptionsWindow:: PackMessage( BMessage * msg )
{
	int32 i32 ;
	switch( fLook.GetSelection() )
	{
		case WB_MiniTab:
			i32 = B_FLOATING_WINDOW_LOOK ;
			break ;

		case WB_Tabless:
			i32 = B_MODAL_WINDOW_LOOK ;
			break ;
	
		case WB_Borderless:
			i32 = B_BORDERED_WINDOW_LOOK ;
			break ;

		case WB_Regular:
		default:
			i32 = B_TITLED_WINDOW_LOOK ;
			break ;
	}
	msg -> AddInt32( "look", i32 ) ;

	switch( fFeel.GetSelection() )
	{
		case WB_Floating :
			i32 = B_FLOATING_ALL_WINDOW_FEEL ;
			break ;
	
		default:
			i32 = B_NORMAL_WINDOW_FEEL ;
			break ;
	}
	msg -> AddInt32( "feel", i32 ) ;

	i32 = 0 ;
	uint32 flags = fFlags.GetValue( ) ;
	if( !(flags & WB_Movable) )
		i32 |= B_NOT_MOVABLE ;
	if( !(flags & WB_Closable) )
		i32 |= B_NOT_CLOSABLE ;
	if( !(flags & WB_Resizable) )
		i32 |= B_NOT_RESIZABLE ;
	if(  (flags & WB_AvoidFront) )
		i32 |= B_AVOID_FRONT ;
	if(  (flags & WB_AvoidFocus) )
		i32 |= B_AVOID_FOCUS ;
	if(  (flags & WB_AcceptFirst) )
		i32 |= B_WILL_ACCEPT_FIRST_CLICK ;
	
	msg -> AddInt32( "flags", i32 ) ;

	bool horizontal = ( fOrientation.GetSelection() == WB_Horizontal ) ;
	msg -> AddBool( "horizontal", horizontal ) ;
}

void WorkBar::OptionsWindow:: MessageReceived( BMessage * msg )
{
	switch( msg->what )
	{
		case MSG_Cancel:
		{
			Cancel() ;
			break ;
		}
		
		case MSG_OK:
		{
			BMessage settings( Messages::SetOptions ) ;
			PackMessage( &settings ) ;
			fTarget.SendMessage( &settings ) ;
			do {  Hide() ; } while( !IsHidden() ) ;
			break ;
		}
		
		case MSG_FilteredKeyDown:
		{
			uint32 key ;
			msg->FindInt32( "raw_char", (int32*)&key ) ;
			if( key == B_ESCAPE )
				Cancel() ;
			break ;
		}
			
		default:
		{		
			inherited::MessageReceived( msg ) ;
			break ;
		}
	}
}

void WorkBar::OptionsWindow:: Cancel( void )
{
	do {  Hide() ; } while( !IsHidden() ) ;
}
