/*
 * RGBColour.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */

#include "RGBColour.h"

#include <Message.h>
#include <Screen.h>

Colours:: RGBColour:: RGBColour( BMessage * msg )
:	BArchivable( msg )
{
	int16 i16 ;

	msg->FindInt16( "red", &i16 ) ;
	red = i16 ;

	msg->FindInt16( "green", &i16 ) ;
	green = i16 ;

	msg->FindInt16( "blue", &i16 ) ;
	blue = i16 ;

	msg->FindInt16( "alpha", &i16 ) ;
	alpha = i16 ;
}

void Colours:: RGBColour:: Merge( const RGBColour& other, float residue )
{
	float altResidue = 1 - residue ;
	if( residue < 0 || altResidue < 0 )
		return ;

	red		= static_cast<uint8>( red 	* residue ) ;
	green	= static_cast<uint8>( green * residue ) ; 
	blue 	= static_cast<uint8>( blue	* residue ) ;
	alpha 	= static_cast<uint8>(alpha	* residue ) ;

	red 	+= static_cast<uint8>(other.red 	* altResidue ) ;
	green 	+= static_cast<uint8>(other.green 	* altResidue ) ;
	blue 	+= static_cast<uint8>(other.blue 	* altResidue ) ;
	alpha 	+= static_cast<uint8>(other.alpha 	* altResidue ) ;
}

status_t Colours:: RGBColour:: Archive( BMessage * msg , bool deep ) const
{
	status_t s = BArchivable::Archive( msg, deep ) ;
	if( s == B_OK )
	{
		msg->AddInt16( "red"	, red	) ;
		msg->AddInt16( "green"	, green	) ;
		msg->AddInt16( "blue"	, blue	) ;
		msg->AddInt16( "alpha"	, alpha	) ;
	}
	return s ;
}


