/*
 * RGBColour.h
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */

#ifndef RGBCOLOUR_H
#define RGBCOLOUR_H

#include  <InterfaceDefs.h>
#include  <Archivable.h>
#include  <stdio.h>

namespace Colours
{
	class RGBColour : public rgb_color , public BArchivable
	{
		public:
		
			BArchivable *	Instantiate(BMessage * ) ;
		 
							RGBColour( void ) ;
							RGBColour( uint8, uint8, uint8, uint8 = 0 ) ;
							RGBColour( const rgb_color& ) ;
							RGBColour( const RGBColour& ) ;
							RGBColour( BMessage * ) ;
							
			status_t		Archive(BMessage *, bool = true) const ;

			void			Tint( float ) ;
			void			Merge( const RGBColour& , float = 0.5) ;
			RGBColour		Inverse( void ) ;
			RGBColour		Polarise( void ) ;
			RGBColour		IPolarise( void ) ;
			
			uint16			Sum( void ) ;
			uint8			Max( void ) ;
			uint8			Min( void ) ;
			float			Luminosity( void ) ;
			
			bool			operator==( const rgb_color& ) ;
			RGBColour&		operator=( const rgb_color& ) ;
			RGBColour&		operator=( const RGBColour& ) ;
			
			void			PrintToStream() ;
			void			PrintToStream( FILE * ) ;
			
		private:
	} ;
}

//===============[ inline functions ]=========================

inline BArchivable * Colours:: RGBColour :: Instantiate( BMessage * msg )
{
	if ( !validate_instantiation(msg, "RGBColour") ) 
		return NULL ; 
	return new RGBColour( msg ) ;
}

inline Colours:: RGBColour:: RGBColour( void )
{
	red 	= (0) ;
	green	= (0) ;
	blue	= (0) ;
	alpha	= (0) ;
}

inline Colours:: RGBColour:: RGBColour( uint8 r, uint8 g, uint8 b, uint8 a )
{
	red 	= (r) ;
	green 	= (g) ;
	blue 	= (b) ;
	alpha	= (a) ;
}

inline Colours:: RGBColour:: RGBColour( const rgb_color& other )
{
	red		= (other.red) ;
	green	= (other.green) ;
	blue	= (other.blue) ;
	alpha	= (other.alpha) ;
}

inline Colours:: RGBColour:: RGBColour( const RGBColour& other )
{
	red		= (other.red) ;
	green	= (other.green) ;
	blue	= (other.blue) ;
	alpha	= (other.alpha) ;
}
	
inline void Colours:: RGBColour:: Tint( float f )
{
	*this = tint_color( *this, f ) ;
}

inline uint16 Colours:: RGBColour:: Sum( void )
{
	return red + green + blue ;
}

inline uint8 Colours:: RGBColour:: Max( void )
{
	if( red > green )
		return ( red > blue ) ? red : blue ;
	return ( green > blue ) ? green : blue ;
}
inline uint8 Colours:: RGBColour:: Min( void )
{
	if( red < green )
		return ( red < blue ) ? red : blue ;
	return ( green < blue ) ? green : blue ;
}

inline float Colours:: RGBColour:: Luminosity( void )
{
	return 0.299 * red + 0.587 * green + 0.114 * blue ;
}

inline bool Colours:: RGBColour:: operator==( const rgb_color& other )
{
	return red == other.red
		&& green == other.green
		&& blue == other.blue
		&& alpha == other.alpha ;
}
	
inline Colours:: RGBColour& Colours:: RGBColour:: operator=( const rgb_color& other )
{
	red 	= other.red ;
	green 	= other.green ;
	blue 	= other.blue ;
	alpha 	= other.alpha ;
	return *this ;
}
inline Colours:: RGBColour& Colours:: RGBColour:: operator=( const RGBColour& other )
{
	red           = other.red ;
	green         = other.green ;
	blue          = other.blue ;
	alpha         = other.alpha ;
	return *this ;
}

inline Colours::RGBColour Colours::RGBColour:: Inverse( void )
{
	return RGBColour ( ~red, ~green, ~blue, alpha ) ;
}

inline Colours::RGBColour Colours::RGBColour:: Polarise( void )
{
	return ( Luminosity() > 100 )
		? RGBColour( 255, 255, 255, 0 )
		: RGBColour(   0,   0,   0, 0 ) ;
}

inline Colours::RGBColour Colours::RGBColour:: IPolarise( void )
{
	return ( Polarise().Inverse() ) ;
}

inline void Colours:: RGBColour:: PrintToStream( FILE * f )
{
	fprintf( f, "RGBColour( %d, %d, %d, %d )\n", red, green, blue, alpha ) ;
}
inline void Colours::RGBColour:: PrintToStream()
{
	PrintToStream( stdout ) ;
}
#endif
