/*
 * Settings.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */


#include <be/storage/FindDirectory.h>
#include <be/storage/Directory.h>
#include <be/storage/Path.h>
#include <be/app/Message.h>
#include <be/storage/File.h>
#include <be/storage/Entry.h>

// #include <err.h>
#include <string.h>
#include <stdio.h>

bool LoadSettings( const char * dir, const char * filename , BMessage * msg )
{
	status_t s ;
	BPath path ;
	s = find_directory( B_USER_SETTINGS_DIRECTORY , &path, true ) ;
	if( s != B_OK )
	{
		printf( "Error: Cannot find settings directory : %s\n",
			strerror( s ) ) ;
		return false ;
	}
	
	path.Append( dir ) ;

	BDirectory loaddir( path.Path() ) ;
	s = loaddir.InitCheck() ;
	if( s != B_OK )
	{
		printf( "Error: Cannot open directory: %s (%s)\n",
			path.Path() , strerror( s ) ) ;
		return false ;
	}

	BFile file( &loaddir, filename , B_READ_ONLY ) ;
	s = file.InitCheck() ;
	if( s != B_OK )
	{
		printf( "Error: Cannot open settings file '%s' : %s\n",
			filename, strerror( s ) ) ;
		return false ;
	}
	
	s = msg->Unflatten( &file ) ;
	if( s != B_OK )
	{
		printf( "Error: Cannot restore settings file '%s' : %s\n",
			filename, strerror( s ) ) ;
		return false ;
	}
	
	return true ;
}
	
status_t SaveSettings(const char * dir, const char * filename , BMessage * msg )
{
	status_t s ;
	BPath path ;
	s = find_directory( B_USER_SETTINGS_DIRECTORY , &path, true ) ;
	if( s != B_OK )
	{
		printf( "Error: Cannot find settings directory : %s\n",
			strerror( s ) ) ;
		return s ;
	}

	BDirectory sett_dir( path.Path() ) ;
	s = sett_dir.InitCheck() ;
	if( s != B_OK )
	{
		printf( "Error: Cannot open settings directory : %s\n",
			strerror( s ) ) ;
		return s ;
	}

	path.Append( dir ) ;
	BEntry ent( path.Path() ) ;
	BDirectory subdir ;
	if( !ent.Exists() )
	{
		s = sett_dir.CreateDirectory( dir, &subdir ) ;
		if( s != B_OK )
		{
			printf( "Error: Cannot open settings directory : %s\n",
				strerror( s ) ) ;
			return s ;
		}
	}
	else
	{
		if( !ent.IsDirectory() )
		{
			printf( "Error: %s exists and is not a directory\n" ,
				path.Path() ) ;
			return B_FILE_EXISTS ;
		}
		entry_ref ref ;
		ent.GetRef( &ref ) ;
		subdir.SetTo( &ref ) ;
	}
	
	BFile file( &subdir, filename, B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE ) ;
	s = file.InitCheck() ;
	if( s != B_OK )
	{
		printf( "Error: Cannot open settings file '%s' : %s\n",
			filename, strerror( s ) ) ;
		return s ;
	}
	
	msg->what = 'Sett' ;
	s = msg->Flatten( &file ) ;
	if( s != B_OK )
	{
		printf( "Error: Cannot save settings file '%s' : %s\n",
			filename, strerror( s ) ) ;
		return s ;
	}
	return B_OK ;
}

