/*
 * Version.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */


#ifndef   VERSION_H
#include "Version.h"
#endif

#include <Roster.h>
#include <Application.h>
#include <AppFileInfo.h>

uint8	Version :: MajorVersion ;
uint8	Version :: MiddleVersion ;
uint8	Version :: MinorVersion ;
char 	Version :: Status ;
uint8	Version :: ReleaseNumber ;
uint16 	Version :: BuildNumber = 1 ;
char 	Version :: BuildDate[] = __DATE__ ;
char 	Version :: BuildTime[] = __TIME__ ;
char	Version :: AppName[] = "WorkBar" ;
char	Version :: ShortInfo[64] ;
char	Version :: LongInfo[256] ;
	
bool Version :: LoadVersionData( void )
{	
	app_info ai ; 
	be_app->GetAppInfo( &ai ) ; 
	
	BFile file ;
	file.SetTo( &ai.ref, B_READ_ONLY ) ;
	
	BAppFileInfo afi ;
	if( afi.SetTo( &file ) != B_OK )
		return false ;

	version_info vi ;
	if( afi.GetVersionInfo( &vi , B_APP_VERSION_KIND ) != B_OK )
		return false ;
		
	MajorVersion = vi.major ;
	MiddleVersion = vi.middle ;
	MinorVersion = vi.minor ;
	switch( vi.variety )
	{
		case 0:
			Status = '#' ;
			break ;

		case 1:
			Status = 'a' ;
			break ;

		case 2:
			Status = 'b' ;
			break ;
			
		case 3:
			Status = '*' ;
		
		case 4:
			Status = '$' ;
			break ;

		case 5:
			Status = '-' ;
			break ;
	}
	ReleaseNumber = vi.internal ;
	
	strcpy( ShortInfo, vi.short_info ) ;
	strcpy( LongInfo, vi.long_info ) ;

	return true ;
}

bool Version :: ReleaseVersionData( void )
{
	return true ;
}

void Version :: PrintVersionInfo( FILE * file )
{
	fprintf( file,
			"%s\nVersion: %d.%d.%d %c%d\nBuild:   %d (%s %s)\n",
				AppName ,
				MajorVersion ,	MiddleVersion , MinorVersion ,
				Status , ReleaseNumber ,
				BuildNumber , BuildDate , BuildTime
	) ;
}

void Version :: PrintVersionInfo( char * str )
{
	sprintf( str,
			"%s\nVersion: %d.%d.%d %c%d\nBuild:   %d (%s %s)\n",
				AppName ,
				MajorVersion ,	MiddleVersion , MinorVersion ,
				Status , ReleaseNumber ,
				BuildNumber , BuildDate , BuildTime
	) ;
}

