/*
 * WBApp.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */


#ifndef   WBWINDOW_H
#include "WBWindow.h"
#endif

#ifndef   MESSAGES_H
#include "Messages.h"
#endif

#ifndef   VERSION_H
#include "Version.h"
#endif

#include <Application.h>
#include <Alert.h>
#include <string.h>

namespace WorkBar
{
	class Application : public BApplication
	{
		typedef BApplication inherited ;
		public:
			Application( void ) ;
			
			void AboutRequested( void ) ;
			void MessageReceived( BMessage * ) ;
			void ArgvReceived( int32, char ** ) ;

			status_t	GetSupportedSuites( BMessage * ) ;
			BHandler *	ResolveSpecifier( BMessage * , int32 , BMessage *,
											int32 , const char * ) ;

		private:

			bool	HandleProperty( const char *, BMessage * ) ;
			void HandleLongOption( char * ) ;

			static  property_info sPropList[] ;
	} ;
}

property_info WorkBar :: Application :: sPropList[] = { 
 	{ "Version", { B_GET_PROPERTY, 0 } ,
		{ B_DIRECT_SPECIFIER , 0 }, "" } ,
 	{ "Workspace",
		{ B_GET_PROPERTY, B_SET_PROPERTY, B_COUNT_PROPERTIES, 0 } ,
		{ B_DIRECT_SPECIFIER , 0 }, "" } ,
 	{ "Workspace_Count",
		{ B_GET_PROPERTY, B_SET_PROPERTY, 0 } ,
		{ B_DIRECT_SPECIFIER , 0 }, "" } ,
} ;


WorkBar :: Application :: Application ( void )
:	BApplication( "application/x-vnd.Zod-WorkBar" )
{
	Version::LoadVersionData( ) ;
	Window * wind = new Window ;
	wind->Show() ;
}

void WorkBar :: Application :: MessageReceived ( BMessage * msg )
{
	switch( msg-> what )
	{
		case Messages::WindowClosed:
			Quit() ;
			break ;

		case B_EXECUTE_PROPERTY:
		case B_SET_PROPERTY:
		case B_GET_PROPERTY:
		case B_COUNT_PROPERTIES:
		{
			int32 idx ;
			BMessage spec ;
			msg->GetCurrentSpecifier( &idx, &spec ) ;

			const char * prop ;
			spec.FindString( "property", &prop ) ;
			
			if( !HandleProperty( prop, msg ) )
				inherited::MessageReceived( msg ) ;			
			break; 
		}
				
		default:
			inherited::MessageReceived( msg ) ;
	}
}
	
void WorkBar :: Application :: AboutRequested ( void )
{
	Version::PrintVersionInfo( stdout ) ;

	char str[256] ;
	sprintf( str, "%s v%d.%d\n\tBy Tim Vernum\n\t<zodsoft@kagi.com>\n",
				Version::AppName ,
				Version::MajorVersion , Version::MiddleVersion ) ;
	BAlert * alert ;
	alert = new BAlert( Version::AppName, str , "What a joke!", NULL, NULL, B_WIDTH_FROM_LABEL ) ;
	alert->Go() ;
}

void WorkBar :: Application :: ArgvReceived( int32 c, char ** v )
{
	int i = 1 ;
	for( ; i<c; i++ )
	{
		if( v[i][0] == '-' )
		{
			switch( v[i][1] )
			{
				case 0:
					break ;
					
				case '-':
				{
					HandleLongOption( v[i]+2 ) ;
					break ;
				}
				
				case 'v':
				{
					Version::PrintVersionInfo( stdout ) ;
					break ;
				}
				
				case 'h':
				{
					fprintf( stderr, "Sorry, no help available\n" ) ;
					break ;
				}
				
				default:
					break ;
			}
		}
	}					
}

void  WorkBar :: Application :: HandleLongOption( char * optn )
{
	if( !strcasecmp( optn, "version" ) )
	{
		Version::PrintVersionInfo( stdout ) ;
	}		
	else if( !strcasecmp( optn, "help" ) )
	{
		fprintf( stderr, "Sorry, no help available\n" ) ;
	}		
}

 
status_t WorkBar :: Application :: GetSupportedSuites( BMessage * msg )
{
	msg->AddString("suites", "suite/vnd.Zod-WorkBar" ); 

	BPropertyInfo propInfo( sPropList); 
	msg->AddFlat("messages", &propInfo); 

	return BHandler::GetSupportedSuites( msg );
}

BHandler * WorkBar :: Application :: ResolveSpecifier(
	BMessage * msg , int32 idx, BMessage * spec, int32 what, const char * prop )
{
	if( strcasecmp( prop, "Version" ) == 0 
	 || strcasecmp( prop, "Workspace" ) == 0 
	 || strcasecmp( prop, "Workspaces" ) == 0
	 || strcasecmp( prop, "WorkspaceCount" ) == 0 
	 || strcasecmp( prop, "Workspace_Count" ) == 0 )
	{
		return this;
	}
	
	return inherited::ResolveSpecifier( msg, idx, spec, what, prop ) ;
}
	
bool WorkBar :: Application :: HandleProperty( const char * prop, BMessage * msg )
{
	if( strcasecmp( prop, "Version" ) == 0 )
	{
		switch( msg->what )
		{
			case B_GET_PROPERTY:
			{
				int32 v	= ( Version::MajorVersion << 24 )
						+ ( Version::MinorVersion << 16 )
						+ ( Version::Status << 8 )
						+ ( Version::ReleaseNumber ) ;

				BMessage reply( B_REPLY ) ;
				reply.AddInt64( "result", v ) ; 
				msg->SendReply( &reply ) ;
				break ;
			}
		}

		return true ;
	}

	if( strcasecmp( prop, "Workspace" ) == 0
	 || strcasecmp( prop, "Workspaces" ) == 0 )
	{
		int32 data ; 
		msg->PrintToStream() ;
		switch( msg->what )
		{
			case B_COUNT_PROPERTIES:
			{
				data = count_workspaces() ;

				BMessage reply( B_REPLY ) ;
				reply.AddInt32( "result", data ) ; 
				msg->SendReply( &reply ) ;
				break ;
			}

			case B_GET_PROPERTY:
			{
				data = current_workspace() + 1;

				BMessage reply( B_REPLY ) ;
				reply.AddInt32( "result", data ) ; 
				msg->SendReply( &reply ) ;
				break ;
			}

			case B_SET_PROPERTY:
			{
				if( msg->FindInt32( "data", &data ) == B_OK )
				{
					activate_workspace( data-1 );

					BMessage reply( B_REPLY ) ;
					reply.AddInt32( "error", 0 ) ; 
					msg->SendReply( &reply ) ;
				}
				break ;
			}

		}

		return true ;
	}

	if( strcasecmp( prop, "WorkspaceCount" ) == 0
	 || strcasecmp( prop, "Workspace_Count" ) == 0 )
	{
		int32 data ; 
		msg->PrintToStream() ;
		switch( msg->what )
		{
			case B_GET_PROPERTY:
			{
				data = count_workspaces() ;
				BMessage reply( B_REPLY ) ;
				reply.AddInt32( "result", data ) ; 
				msg->SendReply( &reply ) ;
				break ;
			}

			case B_SET_PROPERTY:
			{
				if( msg->FindInt32( "data", &data ) == B_OK )
				{
					set_workspace_count( data );

					BMessage reply( B_REPLY ) ;
					reply.AddInt32( "error", 0 ) ; 
					msg->SendReply( &reply ) ;

					BWindow * win = WindowAt(0) ;
					if( win )
						win->PostMessage( Messages::UpdateAllWorkspaces ) ;
				}
				break ;
			}

		}

		return true ;
	}

	return false ;
}

////////////////////////////////////////////////////////
////////////////////	MAIN 	////////////////////////
////////////////////////////////////////////////////////

int main( void )
{
	WorkBar::Application app ;
	app . Run ( ) ;
	
	return 0 ;
}

