#ifndef   WBCONFIG_H
#include "WBConfig.h"
#endif

#ifndef   WBWINDOW_H
#include "WBWindow.h"
#endif

#ifndef   SETTINGS_H
#include "Settings.h"
#endif

#include <Screen.h>
#include <stdio.h>

const char * SettingsDir  = "Zod" ;
const char * SettingsFile = "WorkBar_Settings" ;

WorkBar:: Config:: Config( BWindow * win )
:	fPositions( 'Posn' )
{
	SetToDefault(win) ;
	Load(win) ;
}

void WorkBar:: Config:: SetToDefault( BWindow * win )
{
	// We don't want to have the screen locked for too long...
	// Modern versions of BeOS don't care, but r4 did.
	{
		BScreen screen(win) ;
		fCurrentPosition = screen.Frame() ;
	}
		
	fCurrentPosition . left	    = fCurrentPosition.right - 100 ;
	fCurrentPosition . right    = fCurrentPosition.right - 25 ;
	fCurrentPosition . top		= 50 ;
	fCurrentPosition . bottom	= 150 ;

	fLook = B_FLOATING_WINDOW_LOOK ;
	fFeel = B_NORMAL_WINDOW_FEEL ;	

	fFlags	= B_NOT_ZOOMABLE
			| B_AVOID_FRONT
			| B_WILL_ACCEPT_FIRST_CLICK
			| B_OUTLINE_RESIZE
			;

	fHorizontal = false ;
}

void WorkBar:: Config:: Save()
{
	BMessage msg ;
	Pack( &msg ) ;
	SaveSettings( SettingsDir, SettingsFile, &msg ) ;
}	

void WorkBar:: Config:: Load( BWindow * win )
{
	BMessage msg ;
	if( LoadSettings( SettingsDir, SettingsFile, &msg ) == true )
		Unpack(&msg, win) ;
}

void WorkBar:: Config::UpdateFrom( BWindow * win )
{
	fCurrentPosition	= win->Frame( ) ;
	fLook		= win->Look( ) ;
	fFeel		= win->Feel( ) ;
	fFlags		= win->Flags( ) ;
}	

void WorkBar:: Config::Pack( BMessage * msg )
{
	msg -> AddRect(    "position",			fCurrentPosition ) ;
	msg -> AddMessage( "alt_positions",     &fPositions ) ;
	msg -> AddInt32(   "look",   	 		fLook ) ;
	msg -> AddInt32(   "feel",	  	 		fFeel ) ;
	msg -> AddInt32(   "flags",  	 		fFlags ) ;
	msg -> AddBool(    "horizontal",		fHorizontal ) ; 
}

void WorkBar:: Config::Unpack( BMessage * msg , BWindow * win )
{
	int32 i32 ;
	
	if( msg -> FindRect( "position", &fCurrentPosition ) != B_OK )
		fCurrentPosition = win->Frame( ) ;

	if( msg -> FindMessage( "alt_positions", &fPositions ) != B_OK )
		SavePosition(win) ;

	msg -> FindInt32( "look", &i32 ) ;
	fLook = window_look(i32) ;

	msg -> FindInt32( "feel", &i32 ) ;
	fFeel = window_feel(i32) ;

	msg -> FindInt32( "flags", (int32*)&fFlags ) ;
	fFlags |= B_OUTLINE_RESIZE ;
	fFlags |= B_NOT_ZOOMABLE ;

	msg -> FindBool( "horizontal", &fHorizontal ) ; 
}	

void WorkBar :: Config :: SavePosition( BRect & screen )
{
	int32 width = screen.IntegerWidth() ;
	char str[12];
	sprintf( str, "W:%ld", width/100 ) ;

	if( fPositions.HasRect( str ) )
		fPositions.ReplaceRect( str, fCurrentPosition ) ;
	else
		fPositions.AddRect( str, fCurrentPosition ) ;
}

void WorkBar :: Config :: SavePosition( BWindow * win )
{
	fCurrentPosition	= win->Frame( ) ;

	BRect screenRect ;
	{
		BScreen screen( win ) ;
		screenRect = screen.Frame() ;
	}
	SavePosition( screenRect ) ;
}
