/*
 * WBWindow.cpp
 * Part of the Workbar application
 * (c) 1999, Tim Vernum <zodsoft@kagi.com>
 * Distributed under the "Acknowledged Use Licence" see Doc/Licence
 */


#ifndef   WBWINDOW_H
#include "WBWindow.h"
#endif

#ifndef   MESSAGES_H
#include "Messages.h"
#endif

#ifndef   SETTINGS_H
#include "Settings.h"
#endif

#ifndef   WBVIEW_H
#include "WBView.h"
#endif

#ifndef   OPTIONSWINDOW_H
#include "OptionsWindow.h"
#endif

#include <Application.h>
#include <Screen.h>

#include <stdio.h>
#include <string.h>

property_info WorkBar :: Window :: sPropList[] = { 
 	{ "Orientation", { B_GET_PROPERTY, B_SET_PROPERTY, 0 } ,
		{ B_DIRECT_SPECIFIER , 0 }, "" } ,
} ;

WorkBar :: Window :: Window( void )
:	BWindow( BRect( 50,50,75,100 ) , "WorkBar" , B_BORDERED_WINDOW , 0 , B_ALL_WORKSPACES )
,	fpView( NULL )
,	fbPositionValid( NULL )
,	fpOptionsWindow( NULL )
,	config(this)
{
	{
		int32 w = count_workspaces() ;
		fbPositionValid = new bool[ w ];
		for( int32 i = 0 ; i < w ; i++ )
		{
			fbPositionValid[i] = false ;
		}
	}
	
	config.Load(this) ;
	ImplementConfig( ) ;

	BRect r = Bounds() ;	
	fpView = new View( r, config.fHorizontal ) ;
	AddChild( fpView ) ;

	AddShortcut( 'a', B_OPTION_KEY, new BMessage(B_ABOUT_REQUESTED) ) ;
	AddShortcut( 'o', 0, new BMessage(Messages::Options) ) ;	

	fbPositionValid[ current_workspace() ] = true ;
}

void WorkBar :: Window :: MessageReceived( BMessage * msg )
{
	switch( msg -> what )
	{
		case Messages::Options:
		{
			ShowOptionsWindow( ) ;
			break ;
		}
		case Messages::SetOptions:
		{
			bool h = config.fHorizontal ;
			config.Unpack( msg , this ) ;
			ImplementConfig() ;
			if( h != config.fHorizontal )
				Rotate() ;
			break ;
		}
		case B_ABOUT_REQUESTED:
		{
			be_app->PostMessage( B_ABOUT_REQUESTED ) ;
			break ;
		}
		case Messages::UpdateAllWorkspaces:
		{
			PostMessage( Messages::UpdateAllWorkspaces, fpView ) ;
			break ;
		}
		
		case B_SET_PROPERTY:
		case B_GET_PROPERTY:
		{
			int32 idx ;
			BMessage spec ;
			msg->GetCurrentSpecifier( &idx, &spec ) ;

			const char * prop ;
			spec.FindString( "property", &prop ) ;
			
			if( !HandleProperty( prop, msg ) )
				inherited::MessageReceived( msg ) ;			
			break; 
		}
		
		default:
		{
			inherited::MessageReceived( msg ) ;
			break ;
		}
	}	
}

void WorkBar :: Window :: Quit( void )
{
	config.Save() ;
	be_app->PostMessage( Messages::WindowClosed ) ;
	inherited::Quit( ) ;
}
	
void WorkBar :: Window :: FrameMoved( BPoint )
{
	if( fbPositionValid[ current_workspace() ] )
		config.SavePosition(this) ;
}
	
void WorkBar :: Window :: FrameResized( float , float )
{
	if( fbPositionValid[ current_workspace()  ] )
		config.SavePosition(this) ;
}
	
void WorkBar :: Window :: WorkspaceActivated( int32 ws, bool active )
{
	if( active ) 
	{
		if( !fbPositionValid[ws] )
		{
			FindPosition() ;
			fbPositionValid[ws] = true ;
		}
		PostMessage( Messages::UpdateCurrentWorkspace, fpView ) ;
	}
}

void WorkBar :: Window :: WorkspacesChanged( uint32 o,  uint32 n )
{
	PostMessage( Messages::UpdateAllWorkspaces, fpView ) ;
}

void WorkBar :: Window :: FindPosition( BRect & screen )
{
	int32 width = screen.IntegerWidth() ;
	char str[12];
	sprintf( str, "W:%ld", width/100 ) ;
	
	BRect r ;
	if( config.fPositions.FindRect( str, &r ) == B_OK )
	{
		config.fCurrentPosition = r ;
	}
}

void WorkBar :: Window :: FindPosition( void )
{
	BRect screenRect ;
	{
		BScreen screen( this ) ;
		screenRect = screen.Frame() ;
	}
	FindPosition( screenRect ) ;
	MoveTo( config.fCurrentPosition.left, config.fCurrentPosition . top ) ;
	ResizeTo( config.fCurrentPosition.Width() , config.fCurrentPosition . Height() ) ;
}

void WorkBar :: Window :: ImplementConfig( void )
{
	BRect screenRect ;
	{
		BScreen screen( this ) ;
		screenRect = screen.Frame() ;
	}
	FindPosition( screenRect ) ;
	
	if( !screenRect.Contains( config.fCurrentPosition ) )
	{
		float w = config.fCurrentPosition.Width() ;
		float h = config.fCurrentPosition.Height() ;
		if( screenRect.bottom < config.fCurrentPosition.bottom )
		{
			config.fCurrentPosition.bottom = screenRect.bottom ;
			config.fCurrentPosition.top = config.fCurrentPosition.bottom - h ;
		}
		if( screenRect.right < config.fCurrentPosition.right )
		{
			config.fCurrentPosition.right = screenRect.right ;
			config.fCurrentPosition.left = config.fCurrentPosition.right - w ;
		}
	}
	
	SetLook( config.fLook ) ; 
	SetFeel( config.fFeel ) ;
	SetFlags( config.fFlags ) ;
	MoveTo( config.fCurrentPosition.left, config.fCurrentPosition . top ) ;
	ResizeTo( config.fCurrentPosition.Width() , config.fCurrentPosition . Height() ) ;
}
	
void WorkBar :: Window :: Rotate( void )
{
	BRect screenRect ;
	{
		BScreen screen( this ) ;
		screenRect = screen.Frame() ;
	}
	float h = config.fCurrentPosition.Width() ;
	float w = config.fCurrentPosition.Height() ;

	config.fCurrentPosition.right = config.fCurrentPosition.left + w ;
	config.fCurrentPosition.bottom = config.fCurrentPosition.top + h ;

	if( config.fCurrentPosition.right > screenRect.right )
	{
		config.fCurrentPosition.right = screenRect.right ;
		config.fCurrentPosition.left = config.fCurrentPosition.right - w ;
	}
	else if( config.fCurrentPosition.left < screenRect.left )
	{
		config.fCurrentPosition.left = screenRect.left ;
		config.fCurrentPosition.right = config.fCurrentPosition.left + w ;
	}

	if( config.fCurrentPosition.bottom > screenRect.bottom )
	{
		config.fCurrentPosition.bottom = screenRect.bottom ;
		config.fCurrentPosition.top = config.fCurrentPosition.bottom - h ;
	}
	else if( config.fCurrentPosition.left < screenRect.left )
	{
		config.fCurrentPosition.top = screenRect.top ;
		config.fCurrentPosition.bottom = config.fCurrentPosition.top + h ;
	}

	MoveTo( config.fCurrentPosition.left, config.fCurrentPosition . top ) ;
	ResizeTo( config.fCurrentPosition.Width() , config.fCurrentPosition . Height() ) ;

	BMessage msg( Messages::ResetView ) ;
	msg.AddFloat( "width", config.fCurrentPosition.Width() ) ;
	msg.AddFloat( "height", config.fCurrentPosition.Height() ) ;
	msg.AddBool( "horizontal", config.fHorizontal ) ;
	PostMessage( &msg, fpView ) ;
}		

void WorkBar :: Window :: ShowOptionsWindow( void )
{
	BMessage msg ;
	config.Pack( &msg ) ;

	bool bNew = false ;
	if( !fpOptionsWindow )
	{
		fpOptionsWindow = new OptionsWindow ;
		bNew = true ;
	}
	else
	{
		fpOptionsWindow->Lock() ;
		fpOptionsWindow->SetWorkspaces( B_CURRENT_WORKSPACE ) ;
	}
	
	BPoint pt ;
	uint32 foo ;
	fpView->GetMouse( &pt , &foo ) ;
	fpView->ConvertToScreen( &pt ) ;

	fpOptionsWindow->Init( &msg, BMessenger( this ), pt ) ;	

	if( !bNew )
	{
		fpOptionsWindow->Unlock() ;
	}
		
	fpOptionsWindow->Show() ;
	fpOptionsWindow->Minimize(false) ;
}
 
status_t WorkBar :: Window :: GetSupportedSuites( BMessage * msg )
{
	msg->AddString("suites", "suite/vnd.Zod-WorkBar_Window" ); 

	BPropertyInfo propInfo( sPropList); 
	msg->AddFlat("messages", &propInfo); 

	return BHandler::GetSupportedSuites( msg );
}

BHandler * WorkBar :: Window :: ResolveSpecifier(
	BMessage * msg , int32 idx, BMessage * spec, int32 what, const char * prop )
{
	if( strcasecmp( prop, "Orientation" ) == 0 )
	{
		return this;
	}
	
	return inherited::ResolveSpecifier( msg, idx, spec, what, prop ) ;
}
	
bool WorkBar :: Window :: HandleProperty( const char * prop, BMessage * msg )
{
	if( strcasecmp( prop, "Orientation" ) == 0 )
	{
		switch( msg->what )
		{
			case B_SET_PROPERTY:
			{
				int32 o ;
				if( msg->FindInt32( "data", &o ) != B_OK )
				{
					msg->SendReply( B_MESSAGE_NOT_UNDERSTOOD ) ;
				}
				else
				{
					bool h = config.fHorizontal ;
					config.fHorizontal = ( o ? true : false ) ;

					BMessage reply( B_REPLY ) ;
					reply.AddInt32( "error", 0 ) ; 
					msg->SendReply( &reply ) ;

					if( h != config.fHorizontal )
						Rotate() ;
				}
				break ;
			}

			case B_GET_PROPERTY:
			{
				int32 o = config.fHorizontal ;
				BMessage reply( B_REPLY ) ;
				reply.AddInt32( "result", o ) ; 
				msg->SendReply( &reply ) ;
				break ;
			}
			
			default:
			{
				msg->SendReply( B_MESSAGE_NOT_UNDERSTOOD ) ;
				break ;
			}
		}

		return true ;
	}

	return false ;
}
